@@ -3,20 +3,21 @@
  *
  *      Copyright 2013 Andriy Grytsenko (LStranger) <andrej@rep.kiev.ua>
  *
- *      This program is free software; you can redistribute it and/or modify
- *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation; either version 2 of the License, or
- *      (at your option) any later version.
+ *      This file is a part of libfm-extra package.
  *
- *      This program is distributed in the hope that it will be useful,
+ *      This library is free software; you can redistribute it and/or
+ *      modify it under the terms of the GNU Lesser General Public
+ *      License as published by the Free Software Foundation; either
+ *      version 2.1 of the License, or (at your option) any later version.
+ *
+ *      This library is distributed in the hope that it will be useful,
  *      but WITHOUT ANY WARRANTY; without even the implied warranty of
- *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *      GNU General Public License for more details.
+ *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *      Lesser General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *      MA 02110-1301, USA.
+ *      You should have received a copy of the GNU Lesser General Public
+ *      License along with this library; if not, write to the Free Software
+ *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /**
@@ -210,7 +211,7 @@ FmXmlFileTag fm_xml_file_set_handler(FmXmlFile *file, const char *tag,
     file->tags[i].handler = handler;
     file->tags[i].in_line = in_line;
     file->n_tags = i + 1;
-    g_debug("XML parser: added handler '%s' id %u", tag, (guint)i);
+    /* g_debug("XML parser: added handler '%s' id %u", tag, (guint)i); */
     return i;
 }
 
@@ -594,8 +595,8 @@ _restart:
                                     _("Space isn't allowed in the close tag"));
                 return FALSE;
             }
-            g_debug("XML parser: found closing tag '%s' for %p at %d:%d", tag,
-                    file->current_item, file->line, file->pos);
+            /* g_debug("XML parser: found closing tag '%s' for %p at %d:%d", tag,
+                    file->current_item, file->line, file->pos); */
             item = file->current_item;
             if (item == NULL) /* no tag to close */
             {
@@ -775,7 +776,7 @@ _attr_error:
             g_string_append_len(file->data, text, ptr);
         if (ptr == size) /* still no end of text */
             return TRUE;
-        if(ptr>0) g_debug("XML parser: got text '%s'", file->data->str);
+        /* if(ptr>0) g_debug("XML parser: got text '%s'", file->data->str); */
         if (ptr == 0) ; /* no text */
         else if (file->current_item == NULL) /* text at top level! */
         {
