@@ -2,7 +2,7 @@
  *      fm-file-menu.c
  *
  *      Copyright 2009 PCMan <pcman.tw@gmail.com>
- *      Copyright 2013-2014 Andriy Grytsenko (LStranger) <andrej@rep.kiev.ua>
+ *      Copyright 2013-2015 Andriy Grytsenko (LStranger) <andrej@rep.kiev.ua>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -232,7 +232,10 @@ static gboolean on_key_pressed(GtkWidget *widget, GdkEventKey *event, FmFileMenu
     GdkModifierType mask = 0;
 
     gdk_window_get_device_position (gtk_widget_get_window(widget),
-                                    gtk_get_current_event_device(),
+                                    gdk_device_manager_get_client_pointer(
+                                        gdk_display_get_device_manager(
+                                            gdk_screen_get_display(
+                                                gtk_widget_get_screen(widget)))),
                                     NULL, NULL, &mask);
     if (mask & GDK_SHIFT_MASK)
     {
@@ -253,7 +256,10 @@ static gboolean on_key_released(GtkWidget *widget, GdkEventKey *event, FmFileMen
     GdkModifierType mask = 0;
 
     gdk_window_get_device_position (gtk_widget_get_window(widget),
-                                    gtk_get_current_event_device(),
+                                    gdk_device_manager_get_client_pointer(
+                                        gdk_display_get_device_manager(
+                                            gdk_screen_get_display(
+                                                gtk_widget_get_screen(widget)))),
                                     NULL, NULL, &mask);
     if (!(mask & GDK_SHIFT_MASK))
     {
